/* eslint dot-notation: 0 */
/* eslint prefer-arrow-callback:0 */
import Random from "@reactioncommerce/random";
import { Meteor } from "meteor/meteor";
import { Factory } from "meteor/dburles:factory";
import { assert, expect } from "meteor/practicalmeteor:chai";
import { sinon } from "meteor/practicalmeteor:sinon";
import ReactionError from "@reactioncommerce/reaction-error";
import { getShop } from "/imports/plugins/core/core/server/fixtures/shops";
import Reaction from "/imports/plugins/core/core/server/Reaction";
import * as Collections from "/lib/collections";
import Fixtures from "/imports/plugins/core/core/server/fixtures";

Fixtures();

describe("cart methods", function () {
  const shop = getShop();
  let sandbox;

  beforeEach(function () {
    sandbox = sinon.sandbox.create();
  });

  afterEach(function () {
    sandbox.restore();
  });

  // Required for creating a cart
  after(() => {
    Meteor.users.remove({});
  });

  describe("cart/removeFromCart", function () {
    let account;
    let accountId;
    let user;
    let userId;

    before(function () {
      user = Factory.create("user");
      userId = user._id;
      account = Factory.create("account", { userId });
      accountId = account._id;
    });

    beforeEach(function () {
      Collections.Cart.remove({});
    });

    it("should remove item from cart", function (done) {
      this.timeout(5000);
      const cart = Factory.create("cart", { accountId });
      sandbox.stub(Reaction, "getShopId", () => shop._id);
      const updateSpy = sandbox.spy(Collections.Cart, "update");
      const cartFromCollection = Collections.Cart.findOne({ _id: cart._id });
      const cartItemId = cartFromCollection.items[0]._id;
      assert.equal(cartFromCollection.items.length, 2);
      Meteor.call("cart/removeFromCart", cartItemId);
      // The cart/removeFromCart method will trigger the hook
      // afterCartUpdateCalculateDiscount 2 times.
      assert.equal(updateSpy.callCount, 2, "update should be called two times");
      const updatedCart = Collections.Cart.findOne({ _id: cart._id });
      assert.equal(updatedCart.items.length, 1, "there should be one item left in cart");
      return done();
    });

    it("should decrease the quantity when called with a quantity", function () {
      const cart = Factory.create("cartTwo", { accountId });
      sandbox.stub(Reaction, "getShopId", () => shop._id);
      const cartFromCollection = Collections.Cart.findOne({ _id: cart._id });
      const cartItemId = cartFromCollection.items[0]._id;
      Meteor.call("cart/removeFromCart", cartItemId, 1);
      const updatedCart = Collections.Cart.findOne({ _id: cart._id });
      expect(updatedCart.items[0].quantity).to.equal(1);
    });

    it("should remove cart item when quantity is decreased to zero", function () {
      const cart = Factory.create("cartOne", { accountId });
      sandbox.stub(Reaction, "getShopId", () => shop._id);
      const cartFromCollection = Collections.Cart.findOne({ _id: cart._id });
      const cartItemId = cartFromCollection.items[0]._id;
      const originalQty = cartFromCollection.items[0].quantity;
      Meteor.call("cart/removeFromCart", cartItemId, originalQty);
      const updatedCart = Collections.Cart.findOne({ _id: cart._id });
      expect(updatedCart.items.length).to.equal(0);
    });

    it("should throw an exception when attempting to remove item from cart of another user", function (done) {
      Factory.create("cart");
      const cartItemId = "testId123";

      function removeFromCartFunc() {
        return Meteor.call("cart/removeFromCart", cartItemId);
      }
      expect(removeFromCartFunc).to.throw(ReactionError, /not-found/);
      return done();
    });

    it("should throw an exception when attempting to remove non-existing item", function (done) {
      const cartItemId = Random.id();

      function removeFromCartFunc() {
        return Meteor.call("cart/removeFromCart", cartItemId);
      }
      expect(removeFromCartFunc).to.throw(ReactionError, /not-found/);
      return done();
    });
  });
});
